@@ -1,16 +1,13 @@
 //! Helper module that wraps some Mutex types with different implementations.
-//!
-//! When the `single_threaded` feature is on the mutexes will panic when locked from different threads.
-
-#[cfg(not(any(feature = "single_threaded", feature = "multi_threaded")))]
-compile_error!("Either feature \"single_threaded\" or \"multi_threaded\" must be enabled.");
 
 // ----------------------------------------------------------------------------
 
-#[cfg(feature = "multi_threaded")]
+#[cfg(not(target_arch = "wasm32"))]
 #[cfg(not(debug_assertions))]
 mod mutex_impl {
-    /// Provides interior mutability. Only thread-safe if the `multi_threaded` feature is enabled.
+    /// Provides interior mutability.
+    ///
+    /// Uses `parking_lot` crate on native targets, and `atomic_refcell` on `wasm32` targets.
     #[derive(Default)]
     pub struct Mutex<T>(parking_lot::Mutex<T>);
 
@@ -30,10 +27,12 @@ mod mutex_impl {
     }
 }
 
-#[cfg(feature = "multi_threaded")]
+#[cfg(not(target_arch = "wasm32"))]
 #[cfg(debug_assertions)]
 mod mutex_impl {
-    /// Provides interior mutability. Only thread-safe if the `multi_threaded` feature is enabled.
+    /// Provides interior mutability.
+    ///
+    /// Uses `parking_lot` crate on native targets, and `atomic_refcell` on `wasm32` targets.
     #[derive(Default)]
     pub struct Mutex<T>(parking_lot::Mutex<T>);
 
@@ -111,7 +110,7 @@ mod mutex_impl {
     }
 }
 
-#[cfg(feature = "multi_threaded")]
+#[cfg(not(target_arch = "wasm32"))]
 mod rw_lock_impl {
     /// The lock you get from [`RwLock::read`].
     pub use parking_lot::MappedRwLockReadGuard as RwLockReadGuard;
@@ -119,7 +118,9 @@ mod rw_lock_impl {
     /// The lock you get from [`RwLock::write`].
     pub use parking_lot::MappedRwLockWriteGuard as RwLockWriteGuard;
 
-    /// Provides interior mutability. Only thread-safe if the `multi_threaded` feature is enabled.
+    /// Provides interior mutability.
+    ///
+    /// Uses `parking_lot` crate on native targets, and `atomic_refcell` on `wasm32` targets.
     #[derive(Default)]
     pub struct RwLock<T>(parking_lot::RwLock<T>);
 
@@ -141,18 +142,15 @@ mod rw_lock_impl {
     }
 }
 
-#[cfg(feature = "multi_threaded")]
-mod arc_impl {
-    pub use std::sync::Arc;
-}
-
 // ----------------------------------------------------------------------------
 
-#[cfg(not(feature = "multi_threaded"))]
+#[cfg(target_arch = "wasm32")]
 mod mutex_impl {
     // `atomic_refcell` will panic if multiple threads try to access the same value
 
-    /// Provides interior mutability. Only thread-safe if the `multi_threaded` feature is enabled.
+    /// Provides interior mutability.
+    ///
+    /// Uses `parking_lot` crate on native targets, and `atomic_refcell` on `wasm32` targets.
     #[derive(Default)]
     pub struct Mutex<T>(atomic_refcell::AtomicRefCell<T>);
 
@@ -173,7 +171,7 @@ mod mutex_impl {
     }
 }
 
-#[cfg(not(feature = "multi_threaded"))]
+#[cfg(target_arch = "wasm32")]
 mod rw_lock_impl {
     // `atomic_refcell` will panic if multiple threads try to access the same value
 
@@ -183,7 +181,9 @@ mod rw_lock_impl {
     /// The lock you get from [`RwLock::write`].
     pub use atomic_refcell::AtomicRefMut as RwLockWriteGuard;
 
-    /// Provides interior mutability. Only thread-safe if the `multi_threaded` feature is enabled.
+    /// Provides interior mutability.
+    ///
+    /// Uses `parking_lot` crate on native targets, and `atomic_refcell` on `wasm32` targets.
     #[derive(Default)]
     pub struct RwLock<T>(atomic_refcell::AtomicRefCell<T>);
 
@@ -206,15 +206,8 @@ mod rw_lock_impl {
     }
 }
 
-#[cfg(not(feature = "multi_threaded"))]
-mod arc_impl {
-    // pub use std::rc::Rc as Arc; // TODO(emilk): optimize single threaded code by using `Rc` instead of `Arc`.
-    pub use std::sync::Arc;
-}
-
 // ----------------------------------------------------------------------------
 
-pub use arc_impl::Arc;
 pub use mutex_impl::{Mutex, MutexGuard};
 pub use rw_lock_impl::{RwLock, RwLockReadGuard, RwLockWriteGuard};
 
